from __future__ import division

class TS(object):
    """A class for representing time series.

    Remarkably close to just being a list.  But has mean() and median().

    Set accuracy = 3 to get rounding to third decimal.
    """

    def __init__(self, data=None, accuracy=None):
        self.data = []
        self.accuracy = accuracy
        if data:
            self.data = [x for x in data]

    def __len__(self):
        return len(self.data)

    def __getitem__(self, idx):
        return self._round(self.data[idx])

    def append(self, d):
        self.data.append(d)

    def __repr__(self):
        return str(self)

    def _round(self, val):
        if self.accuracy is None:
            return val
        return round(val, self.accuracy)

    def _roundall(self, data):
        if self.accuracy is None:
            return data
        return map(self._round, data)

    def __str__(self):
        if len(self) < 10:
            return str(self._roundall(self.data))

        tmp = self.data[:4]
        tmp += self.data[-4:]
        tmp = self._roundall(tmp)
        fmt = 'TS[{}, {}, {}, {}, ..., {}, {}, {}, {}]'
        return fmt.format(*tmp)

    def mean(self):
        if len(self) == 0:
            raise ValueError('Arithmetic mean of empty time series.')
        m = sum(self.data) / float(len(self))
        return self._round(m)

    def median(self):
        if len(self) == 0:
            raise ValueError('Median of empty time series.')
        tmp = sorted(self.data)
        return self._round(tmp[len(self)//2])

    def stddev(self):
        ld = len(self.data)
        if ld < 2:
            raise ValueError('Standard deviation of too few values')
        m = self.mean()
        ss = sum((x-m)**2 for x in self.data)
        std = (ss/float(ld))**0.5
        return self._round(std)



from __future__ import print_function

import sys
from os import makedirs
from os.path import abspath, isfile, isdir, split
import sqlite3
from datetime import datetime as dt
from dateutil import parser as dt_parser


def _insert(val, sensor):
    """Returns INSERT query"""
    # TODO make this prepared statement
    query = "INSERT INTO samples (id, value, sensor, timestamp) VALUES (NULL, %f, '%s', '%s');"
    return query % (val, sensor, dt.utcnow())


class FriskbyDao(object):
    """The *FriskbyDao* is a data access object that currently persists measured
    data to an SQLITE file using the `sqlite3` Python module.

    The underlying database _scheme_ is simple:

    ```sql
    CREATE TABLE samples (
        `id` INTEGER PRIMARY KEY,
        `value` FLOAT NOT NULL,
        `sensor` TEXT NOT NULL,
        `timestamp` TEXT NOT NULL,
        `uploaded` BOOL DEFAULT 0
        );
    ```

    The `sensor` describes contains the sensor ID and describes what type of
    measurement has been done.  The value is its value (in an assumed known (SI)
    unit).  The `uploaded` flag signifies whether the measurement has been
    uploaded

    """
    # TODO make FriskbyDao a context manager so we can:
    # with FriskbyDao(path) as dao:
    #    dao.get_non_uploaded(...)
    #    submitter.post(...)
    #    dao.mark_uploaded(...)

    def __init__(self, sql_path):
        """The sqlite db has a table called 'samples' with schema
        id, value, sensor, timestamp, uploaded
        """
        self._sql_path = abspath(sql_path)
        self.__init_sql()

    def get_path(self):
        return self._sql_path

    def __init_sql(self):
        if isfile(self._sql_path):
            return # nothing to do

        _path, _ = split(self._sql_path)
        if not isdir(_path):
            makedirs(_path) # equivalent to `mkdir -p _path`
        if not isfile(self._sql_path):
            print('No database, constructing new: %s.' % self._sql_path)
            _id = '`id` INTEGER PRIMARY KEY'
            _val = '`value` FLOAT NOT NULL'
            _sen = '`sensor` TEXT NOT NULL'
            _date = '`timestamp` TEXT NOT NULL'
            _upl = '`uploaded` BOOL DEFAULT 0'
            _create = 'CREATE TABLE samples (%s, %s, %s, %s, %s);'
            schema = _create % (_id, _val, _sen, _date, _upl)
            conn = sqlite3.connect(self._sql_path)
            conn.execute(schema)
            conn.close()

    def get_num_rows(self, uploaded_status=None):
        """Gets num rows in sql storage.  If uploaded_status is set to True, we
        fetch number of rows that are marked uploaded, if uploaded_status is set
        to False, we fetch number of rows that are marked as not uploaded.
        """
        query = 'SELECT COUNT(uploaded) FROM samples'
        if uploaded_status is not None:
            if uploaded_status:
                query += ' WHERE uploaded'
            else:
                query += ' WHERE NOT uploaded'
        query += ';'
        conn = sqlite3.connect(self._sql_path)
        result = conn.execute(query)
        num = result.fetchone()[0]
        conn.close()
        return num


    def get_recent_samples(self, limit=20, uploaded=None):
        """If uploaded=True, get uploaded, if uploaded=False, get non-uploaded.  If
        uploaded=None, ignore upload status.

        """

        sub_q = "id, value, sensor, datetime(timestamp, 'localtime'), uploaded"
        query = 'SELECT %s FROM samples %s ORDER BY id DESC LIMIT %d;'
        upl_q = ''
        if uploaded is True:
            upl_q = 'WHERE `uploaded`'
        if uploaded is False:
            upl_q = 'WHERE NOT `uploaded`'

        conn = sqlite3.connect(self._sql_path)
        result = conn.execute(query % (sub_q, upl_q, limit))
        data = result.fetchall()
        conn.close()
        for i in range(len(data)):
            id_, val_, sens_, dt_, upl_ = data[i]
            data[i] = id_, val_, sens_, dt_parser.parse(dt_), upl_
        return data


    def get_non_uploaded(self, limit=20):
        return self.get_recent_samples(limit=limit, uploaded=False)

    def persist_ts(self, data):
        """Save data to underlying storage.

        Data should be on form {'PM10': TS, 'PM25': TS, ...}, where the TS are
        timeseries, that is, they should have a median: TS->float function.

        """
        queries = []
        for sensor, ts in data.items():
            queries.append(_insert(ts.median(), sensor))
        conn = sqlite3.connect(self._sql_path)
        for query in queries:
            conn.execute(query)
        conn.commit()
        conn.close()

    def last_entry(self, uploaded=None):
        """Returns timestamp of the last entry in the dao, or None if no entry matches
        the criterion.  If uploaded is non-None, then a truthy value will return the
        last uploaded, and a falsy value will give the last non-uploaded entry.
        """
        query_pre = 'SELECT timestamp FROM samples'
        query_mid = ' WHERE `uploaded` = %d' % (1 if uploaded else 0)
        query_post = ' ORDER BY timestamp DESC LIMIT 1'
        query = query_pre + query_post
        return_value = None
        if uploaded is not None:
            query = query_pre + query_mid + query_post
        conn = sqlite3.connect(self._sql_path)
        result = conn.execute(query)
        data = result.fetchall()
        if data:
            return_value = dt_parser.parse(data[0][0])
        conn.close()
        return return_value


    def mark_uploaded(self, data):
        print('dao marking ...')
        sys.stdout.flush()
        query = 'UPDATE samples SET uploaded=1 WHERE id=%s'

        conn = sqlite3.connect(self._sql_path)
        conn.execute('begin')
        for row in data:
            # id, value, sensor, timestamp, uploaded
            id_ = row[0]
            conn.execute(query % id_)
        conn.commit()
        conn.close()


    def __repr__(self):
        try:
            num, num_up = self.get_num_rows(), self.get_num_rows(uploaded_status=True)
            fmt = 'FriskbyDao(num_rows=%s, non_uploaded=%s, path=%s)'
            return fmt % (num, num_up, self._sql_path)
        except:
            return 'FriskbyDao(path=%s)' % (self._sql_path)


if __name__ == '__main__':
    if len(sys.argv) > 1:
        print(FriskbyDao(sys.argv[1]))
    else:
        sys.exit('Need a path to an sqlite database.')



#!/usr/bin/env python
from __future__ import (print_function, absolute_import)

import json
import sys
import time

from .os_release import sys_info


SYSTEMD_NAME = "org.freedesktop.systemd1"
SYSTEMD_OBJ = "/org/freedesktop/systemd1"
SYSTEMD_UNIT_IFACE = "org.freedesktop.systemd1.Unit"
DBUS_PROPERTIES_IFACE = "org.freedesktop.DBus.Properties"
SYSTEMD_MANAGER_IFACE = "org.freedesktop.systemd1.Manager"


class FriskbyManager(object):

    def __init__(self, device_config, dbus=None, pip=None,
                 managed_packages=None, managed_services=None):
        """Creates a Friskby manager. Given a config, the manager will upgrade
        the client's pip packages, and appropriately reload any systemd
        units it manages.

        managed_packages is a list of pip packages that it will upgrade
            (according to the config's channel value).
        managed_services is a list of systemd services.
        """
        self._config = device_config
        self._dbus = dbus
        self._pip = pip
        self._managed_packages = managed_packages
        self._managed_services = managed_services

    def __run_post_install(self):
        """
        Checks if any known service needs a daemon-reload. If a service needs
        it, we do a daemon-reload.
        """
        if self._dbus is None:
            print("No DBus provided, post-install can't run.")
            sys.stdout.flush()
            return

        if len(self._managed_services) == 0:
            print("No services to manage.")
            sys.stdout.flush()
            return

        sysbus = self._dbus.SystemBus()
        systemd1_obj = sysbus.get_object(SYSTEMD_NAME, SYSTEMD_OBJ)
        manager = self._dbus.Interface(systemd1_obj, SYSTEMD_MANAGER_IFACE)

        needs_daemon_reload = []
        units_to_be_restarted = []

        for unit in self._managed_services:
            unit_obj_path = manager.GetUnit(unit)
            unit_obj = sysbus.get_object(SYSTEMD_NAME, unit_obj_path)

            # Creates two unit proxies, one for issuing commands, and another
            # for retrieving properties.
            props_proxy = self._dbus.Interface(unit_obj, DBUS_PROPERTIES_IFACE)
            unit_proxy = self._dbus.Interface(unit_obj, SYSTEMD_UNIT_IFACE)

            need_reload = props_proxy.Get(SYSTEMD_UNIT_IFACE,
                                          "NeedDaemonReload")
            active_state = props_proxy.Get(SYSTEMD_UNIT_IFACE,
                                           "ActiveState")
            names = props_proxy.Get(SYSTEMD_UNIT_IFACE,
                                    "Names")
            if need_reload:
                print("Unit %s required daemon-reload." % unit)
                sys.stdout.flush()
                needs_daemon_reload.append(unit)

            # Queue units to be restarted if active, but is not this service.
            if active_state == "active" and "friskby.service" not in names:
                print("Unit %s was active and will be restarted." % unit)
                sys.stdout.flush()
                units_to_be_restarted.append(unit_proxy)

        # TODO: Instead of sleeping, we could track the job and wait for its
        # completion.
        if len(needs_daemon_reload) > 0:
            manager.Reload()
            time.sleep(5)

        for up in units_to_be_restarted:
            up.Restart("replace")

    def install(self, config):
        # TODO should we do pip install --upgrade reuirements from config?
        config.save()

    def update_client(self, config):
        """Updates or upgrades the client's pip packages.

        Raises RuntimeError should pip return a non-zero exit code.
        """
        if self._pip is None:
            print("No pip provided, post install can't run.")
            sys.stdout.flush()
            return

        if len(self._managed_packages) == 0:
            print("No packages to manage.")
            sys.stdout.flush()
            return

        new_config = config.downloadNew()

        channel = "stable"
        try:
            channel = new_config.getChannel()
        except KeyError:
            pass  # No channel defined, use stable.

        args = ["install", "--upgrade"]
        if channel == "latest":
            args.append("--pre")
        args = args + self._managed_packages

        ret = self._pip.main(args=args)
        if ret == 0:
            self.__run_post_install()
        else:
            raise RuntimeError("Pip exited with non-zero code: %d" % ret)

    def run(self):
        # TODO this logic must be rewritten
        long_msg = self.get_sys_info()
        self._config.logMessage("Starting up", long_msg=long_msg)
        self._config.postVersion()

        self.update_client(self._config)

    @classmethod
    def get_sys_info(cls):
        info = {}
        try:
            info = sys_info()
            if info:
                info = json.dumps(info, indent=4, sort_keys=True)
        except Exception as err:
            info = 'Error getting sys_info: "%s".' % err
        return info



#!/usr/bin/env python
from __future__ import print_function

import sys
import json
import requests

class FriskbySubmitter(object):
    """This class submits data from a database to the friskby cloud, then proceeds
    to mark the uploaded data as such.

    """

    def __init__(self, dao, config):
        self.device_config = config
        self.dao = dao

    def _upload(self, rows):
        if not rows:
            return
        print('Attempting to upload.')
        sys.stdout.flush()
        # id, value, sensor, timestamp, uploaded
        data = {}
        for row in rows:
            id_, value, sensor, timestamp, _ = row
            if sensor not in data:
                data[sensor] = []
            data[sensor].append((id_, value, sensor, timestamp))

        print('Connecting.')
        sys.stdout.flush()
        for sensor in data:
            sensor_id = self.device_config.getSensorId(sensor)
            push = {"sensorid"   : sensor_id,
                    "value_list" : [(x[3].isoformat(), x[1]) for x in data[sensor]], # (time, val)
                    "key"        : self.device_config.getPostKey()}
            print('Posting to %s' % self.device_config.getPostURL())
            print(push)
            sys.stdout.flush()
            respons = requests.post(self.device_config.getPostURL(),
                                    headers={'Content-Type': 'application/json'},
                                    data=json.dumps(push),
                                    timeout=30)
            print('posted %s' % sensor)
            sys.stdout.flush()
            if respons.status_code != 201:
                respons.raise_for_status()
                raise Exception('Server did not respond with 201 Created.  Response: %d %s'
                                % (respons.status_code, respons.text))
            respons.connection.close()
        return True # no exception, everything written

    def post(self):
        """Posts non-uploaded entries from the dao to the cloud.  Marks as
           uploaded in the dao if successfully uploaded."""
        if self.device_config is None:
            raise ValueError('Device config not set!')
        to_upload = self.dao.get_non_uploaded()
        print('Submitting ...')
        # TODO this is not particularly safe!
        # get_non_uploaded should at the very least be a context manager.
        sys.stdout.flush()
        if self._upload(to_upload):
            self.dao.mark_uploaded(to_upload)



"""The sensors submodule of friskby.

Contains drivers for all known sensors, currently only SDS011, though.
"""
from __future__ import absolute_import

from .sds011 import SDS011

__all__ = ['SDS011']



from __future__ import print_function

from sys import version as sys_version
from os import uname
from os.path import isfile
import json
from datetime import datetime as dt

def _read_os_release(pfx='LSB_'):
    fname = '/etc/os-release'
    if not isfile(fname):
        return {}
    def processline(ln):
        return ln.strip().replace('"', '')
    def splitline(ln, pfx=''):
        if ln.count('=') == 1:
            k, v = ln.split('=')
            return pfx+k, v
        return None
    props = {}
    with open(fname, 'r') as f:
        for line in f:
            kv = splitline(processline(line), pfx=pfx)
            if kv:
                props[kv[0]] = kv[1]
    return props

def sys_info():
    """This gets basic system information to log on startup"""
    sysname, nodename, release, version, machine = uname()
    python_vs, python_cc = sys_version.split('\n')
    req_vs = '0.0.0'
    local_time = dt.now().isoformat()
    lsb = None
    try:
        lsb = _read_os_release()
    except:
        pass
    try:
        import requests
        req_vs = requests.__version__
    except:
        pass
    if not lsb:
        lsb = {}
    sysinf = {'sysname'  : sysname,
              'nodename' : nodename,
              'machine'  : machine,
              'release'  : release,
              'version'  : version,
              'python'   : python_vs,
              'pythoncc' : python_cc,
              'requests' : req_vs,
              'localtime': local_time}
    sysinf.update(lsb)
    return sysinf

if __name__ == '__main__':
    print(json.dumps(sys_info(), indent=4, sort_keys=True))



"""The friskby module.

The _friskby_ module is a Python module for reading from external sensors
(currently, only the `SDS011` sensor is supported), store measurements to a
temporary database and post the measurements to a webserver.

In the future we will add other sensors for temperature, relative humidity,
barometric pressure, noise level detection, luminosity, etc, and will post data
for such sensors to a webserver.

# Overview of functionality

The module consists of three primary features:

1. sampling
2. submitting
3. updating

# FriskbySampler

The FriskbySampler is a module that connects to the sensor and samples
information about the weather, air, climate or surrounding environment, and
stores the data to the *FriskbyDao*.  The *FriskbyDao* is a data access object
that currently persists measured data to an SQLITE file using the `sqlite3`
Python module.

The underlying database _scheme_ is simple:

```sql
CREATE TABLE samples (
    `id` INTEGER PRIMARY KEY,
    `value` FLOAT NOT NULL,
    `sensor` TEXT NOT NULL,
    `timestamp` TEXT NOT NULL,
    `uploaded` BOOL DEFAULT 0
    );
```

The `sensor` describes contains the sensor ID and describes what type of
measurement has been done.  The value is its value (in an assumed known (SI)
unit).  The `uploaded` flag signifies whether the measurement has been uploaded,
which is dealt with by the

# FriskbySubmitter

The FriskbySubmitter reads non-uploaded measurements from the FriskbyDao and
submits these measurements to the prescribed webserver.  The URL and API key for
the webserver is defined in a config file provided by the caller.  The submitter
uses the `requests` module in Python (it is recommended that one has
`requests>=2.13.0`, which is the latest one).

# FriskbyManager

The job of the FriskbyManager is to read the aforementioned config file, contact
the webserver and ask for today's news.  They are rare, but may contain
reconfiguration of API keys, URL's and occasionally requests that the client
updates itself.

"""

from __future__ import absolute_import

from serial import SerialException
from .device_config import DeviceConfig
from .friskby_dao import FriskbyDao
from .friskby_sampler import FriskbySampler
from .friskby_manager import FriskbyManager
from .friskby_submitter import FriskbySubmitter
from .os_release import sys_info
from .ts import TS

from .sensors import SDS011

VERSION = '0.72.1'
__all__ = ['FriskbyDao', 'FriskbySampler', 'FriskbySubmitter', 'FriskbyManager',
           'TS', 'DeviceConfig', 'sys_info']

__version__ = VERSION



#!/usr/bin/env python
from __future__ import division, print_function
from os.path import exists
import time
from serial import Serial, SerialException


class SDS011(object):
    """A class for reading PM10 and PM25 from the SDS011 sensor.'

    Takes as input the path to the device, e.g. 'dev/ttyUSB0' or
    '/dev/ttyAMA0'.  Has one method, read, which reads for

    """
    MSG_START = 170  # AAC0
    MSG_CMD = 192
    MSG_END = 171

    def __init__(self, device_path, sleep_time=0.01):
        self.sleep_time = sleep_time
        if not exists(device_path):
            raise IOError('[Errno 2] No such device %s' % device_path)
        self.device = Serial(device_path, baudrate=9600,
                             stopbits=1, parity="N", timeout=2)

    def _fastforward(self):
        """Reads from device until [MSG_START=AAC0, MSG_CMD] encountered"""
        while True:
            byte_read = self.device.read(1)
            if len(byte_read) < 1:
                raise SerialException('Device timed out in attempt to read a byte.')
            if ord(byte_read) == SDS011.MSG_START:
                byte_read = self.device.read(1)
                if ord(byte_read) == SDS011.MSG_CMD:
                    return
            time.sleep(self.sleep_time)

    def _read_values(self):
        """Reads 8 values as bytes and their int values"""
        bytes_read = self.device.read(8)
        if len(bytes_read) != 8:
            raise SerialException('Device timed out in attempt to read values.')
        return map(ord, bytes_read)

    def read(self):
        """Samples from sensor, verifies correctness and returns a pair
        of float values (pm10, pm25)."""
        self._fastforward()
        pm25hb, pm25lb, pm10hb, pm10lb, d5, d6, cs, tail = self._read_values()

        cs_expected = (pm25hb + pm25lb + pm10hb + pm10lb + d5 + d6) % 256
        if cs != cs_expected:
            raise Exception("Checksum test failed")

        if tail != SDS011.MSG_END:
            raise Exception("Message was not correctly terminated?")

        pm25 = float(pm25hb + pm25lb*256)/10.0
        pm10 = float(pm10hb + pm10lb*256)/10.0

        return (pm10, pm25)

if __name__ == '__main__':
    import sys
    PATH = '/dev/ttyUSB0'
    if len(sys.argv) == 2:
        PATH = sys.argv[1]

    SENSOR = None
    try:
        SENSOR = SDS011(PATH)
    except IOError as i_err:
        if '[Errno 13]' in str(i_err):
            sys.exit('[Errno 13] Permission denied for %s.  Try with sudo.' % PATH)
        if '[Errno 2]' in str(i_err):
            sys.exit('No such device %s. Try another path or port.' % PATH)
        sys.exit('Failed to connect to device at %s.\n%s' % (PATH, i_err))
    if not SENSOR:
        sys.exit('Failed to connect to %s.' % PATH)

    try:
        PM10PM25 = SENSOR.read()
        print('%.2f %.2f' % PM10PM25)
    except SerialException as s_err:
        sys.exit('Error during sampling: %s.' % s_err)



from setuptools import setup

DEPENDENCIES = ['requests', 'pyserial', 'python-dateutil', 'pip']

setup(
    name='friskby',
    version='0.72.1',
    description='The friskby module',
    url='http://github.com/FriskbyBergen/python-friskby',
    author='Friskby Bergen',
    keywords='friskby bergen frisk by python-friskby a-small-code air',
    author_email='pgdr@statoil.com',
    license='GNU General Public License, Version 3',
    packages=['friskby', 'friskby/sensors'],
    zip_safe=False,
    setup_requires=DEPENDENCIES,
    install_requires=DEPENDENCIES,
    tests_require=['pylint'],
    test_suite='tests',
)



from __future__ import print_function

import sys
import time
from datetime import datetime as dt

from .ts import TS

class FriskbySampler(object):
    """This class is initialized with a reader (a sensor of type SDS011), a dao and
    sample time.  The collect method collects data for `sample_time` amount of
    time, and then asks the FriskbyDao to persist this data.

    """

    def __init__(self, reader, dao, sample_time, sleep_time=0.10, accuracy=None):
        """
        Takes a reader (sensor) and a dao, and collects data and persists to dao.
        """
        self.reader = reader
        self.sample_time = sample_time
        self.sleep_time = sleep_time
        self.accuracy = accuracy
        self.dao = dao

    def collect(self):
        """Reads values from the sensor and writes to dao.
        """
        # reader/sds011 returns (PM10, PM25)
        data = (TS(accuracy=self.accuracy), TS(accuracy=self.accuracy))
        print('\tShh, I am collecting.')
        sys.stdout.flush()
        start = dt.now()
        while True:
            pm10, pm25 = self.reader.read()
            data[0].append(pm10)
            data[1].append(pm25)

            dt_now = dt.now() - start
            if dt_now.total_seconds() >= self.sample_time:
                break
            time.sleep(self.sleep_time)
        print('\tDone collecting, storing and returning.')
        sys.stdout.flush()
        samples = {'PM10': data[0], 'PM25': data[1]}
        self.dao.persist_ts(samples)



from __future__ import absolute_import
import os.path
from sys import stderr

import json
import datetime
import tempfile
from urlparse import urlparse
import requests
import friskby

class DeviceConfig(object):
    required_keys = ["post_key", "sensor_list", "post_path", "config_path",
                     "server_url", "device_id"]

    def __init__(self, filename, post_key=None):
        if not os.path.isfile(filename):
            raise IOError("No such file: %s" % filename)

        config = None
        with open(filename, 'r') as js:
            config = json.load(js)
        if not "post_key" in config:
            if not post_key is None:
                config["post_key"] = post_key

        for key in self.required_keys:
            if not key in config:
                raise KeyError("Missing key:%s" % key)

        self.sha = None
        self.filename = filename
        self.data = config
        self.config_ts = datetime.datetime.now()

    def get_version(self):
        return friskby.VERSION

    def __eq__(self, other):
        return self.data == other.data

    def __ne__(self, other):
        return not self == other

    def save(self, filename=None):
        if filename is None:
            filename = self.filename

        path = os.path.dirname(filename)
        if not os.path.isdir(path):
            os.makedirs(path)

        with open(filename, "w") as fileH:
            fileH.write(json.dumps(self.data))

    def getServerURL(self):
        return self.data["server_url"]

    def getSensorId(self, sensor_type):
        vals = self.data['sensor_list']
        for s_id in vals:
            if sensor_type in s_id:
                return s_id
        return None

    def getConfigPath(self):
        return self.data["config_path"]

    def getPostURL(self):
        return "%s/%s" % (self.data["server_url"], self.data["post_path"])

    def getDeviceID(self):
        return self.data["device_id"]

    def getPostKey(self):
        return self.data["post_key"]

    def getChannel(self):
        return self.data["channel"]

    def updateRequired(self, new_config):
        return True  # TODO add logic

    def downloadNew(self):
        self.config_ts = datetime.datetime.now()
        update_url = "%s/%s" % (self.getServerURL(), self.getConfigPath())
        new_config = self.download(update_url, post_key=self.getPostKey())
        return new_config


    @classmethod
    def download(cls, url, post_key=None):
        response = requests.get(url, timeout=10)
        if response.status_code != 200:
            raise ValueError("http GET return status:%s" % response.status_code)

        _, config_file = tempfile.mkstemp()
        data = json.loads(response.content)
        tmp = urlparse(url)
        config = data["client_config"]
        config["server_url"] = "%s://%s" % (tmp.scheme, tmp.netloc)
        with open(config_file, "w") as f:
            f.write(json.dumps(config))

        config = DeviceConfig(config_file, post_key=post_key)
        os.unlink(config_file)
        config.filename = None

        return config

    def logMessage(self, msg, long_msg=None):
        data = {"key"     : self.getPostKey(),
                "device_id" : self.getDeviceID(),
                "msg" : msg}
        if long_msg:
            data["long_msg"] = long_msg
        json_msg = json.dumps(data, sort_keys=True)

        stderr.write('DeviceConfig.logMessage: %s\n' % json_msg)

        headers = {"Content-Type": "application/json"}
        response = requests.post("%s/sensor/api/client_log/" % self.getServerURL(),
                                 data=json_msg,
                                 headers=headers, timeout=10)
        return response.status_code

    def postVersion(self):
        data = {"key"     : self.getPostKey(),
                "git_ref" : "%s" % self.get_version()}
        headers = {"Content-Type": "application/json"}

        requests.put("%s/sensor/api/device/%s/" %
                     (self.getServerURL(), self.getDeviceID()),
                     data=json.dumps(data),
                     headers=headers,
                     timeout=10)

    def __repr__(self):
        content = '{}'
        try:
            content = json.dumps(self.data)
        except ValueError:
            pass
        return 'DeviceConfig(%s)' % content
